__title__ = 'fobi.contrib.plugins.form_elements.content.content_video.forms'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('ContentVideoForm',)

from django import forms
from django.utils.translation import ugettext_lazy as _

from fobi.base import BasePluginForm, get_theme

from .settings import DEFAULT_SIZE, SIZES

theme = get_theme(request=None, as_instance=True)

class ContentVideoForm(forms.Form, BasePluginForm):
    """
    Form for ``ContentVideoPlugin``.
    """
    plugin_data_fields = [
        ("title", ""),
        ("url", ""),
        ("size", DEFAULT_SIZE),
    ]

    title = forms.CharField(
        label = _("Title"),
        required = True,
        widget = forms.widgets.TextInput(attrs={'class': theme.form_element_html_class})
        )
    url = forms.CharField(
        label = _("URL"),
        required = True,
        widget = forms.widgets.TextInput(attrs={'class': theme.form_element_html_class})
        )
    size = forms.ChoiceField(
        label = _("Size"),
        required = False,
        initial = DEFAULT_SIZE,
        choices = SIZES,
        widget = forms.widgets.Select(attrs={'class': theme.form_element_html_class})
        )



__title__ = 'fobi.contrib.plugins.form_elements.fields.checkbox_select_multiple'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('default_app_config', 'UID',)

default_app_config = 'fobi.contrib.plugins.form_elements.fields.checkbox_select_multiple.apps.Config'

UID = 'checkbox_select_multiple'



__title__ = 'fobi.tests.test_dynamic_forms'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__al__ = ('FobiDynamicFormsTest',)

import unittest

from django.test import TestCase

from fobi.dynamic import assemble_form_class
from fobi.tests.base import print_info
from fobi.tests.helpers import (
    setup_fobi, get_or_create_admin_user, create_form_with_entries
)

class FobiDynamicFormsTest(TestCase):
    """
    Tests of django-fob dynamic forms functionality.
    """
    def setUp(self):
        setup_fobi(fobi_sync_plugins=True)
        user = get_or_create_admin_user()
        create_form_with_entries(user)

    @print_info
    def test_01_assemble_form_class_and_render_form(self):
        """
        Test form class assembling and rendering.
        """
        flow = []

        # Getting entry with created plugins
        form_entry = create_form_with_entries()
        flow.append(form_entry)

        FormClass = assemble_form_class(form_entry)
        flow.append(FormClass)

        form = FormClass()
        flow.append(form)

        rendered_form = form.as_p()
        flow.append(rendered_form)

        return flow


if __name__ == '__main__':
    unittest.main()



__title__ = 'fobi.contrib.plugins.form_handlers.db_store.admin'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('SavedFormDataEntryAdmin',)

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from .models import SavedFormDataEntry
from .helpers import DataExporter

class SavedFormDataEntryAdmin(admin.ModelAdmin):
    """
    Saved form data entry admin.
    """
    list_display = ('form_entry', 'user', 'formatted_saved_data', 'created',)
    list_filter = ('form_entry', 'user',)
    readonly_fields = ('created', 'formatted_saved_data')
    fieldsets = (
        (None, {
            'fields': ('form_entry', 'user',)
        }),
        (_("Data"), {
            'fields': ('formatted_saved_data', 'created',)
        }),
        (_("Raw"), {
            'classes': ('collapse',),
            'fields': ('form_data_headers', 'saved_data',)
        }),
    )

    actions = ['export_data']

    class Meta:
        app_label = _('Saved form data entry')

    class Media:
        js = (
            '{0}js/jquery-1.10.2.min.js'.format(settings.STATIC_URL),
            '{0}db_store/js/db_store.js'.format(settings.STATIC_URL),
            '{0}db_store/js/jquery.expander.min.js'.format(settings.STATIC_URL),
            )

    def queryset(self, request):
        queryset = super(SavedFormDataEntryAdmin, self).queryset(request)
        #queryset = queryset.select_related('form_entry', 'user',)
        return queryset
    get_queryset = queryset

    def export_data(self, request, queryset):
        """
        Export data into XLS.
        """
        data_exporter = DataExporter(queryset)

        return data_exporter.graceful_export()

    export_data.short_description = _('Export data to CSV/XLS')


admin.site.register(SavedFormDataEntry, SavedFormDataEntryAdmin)



__title__ = 'fobi.contrib.plugins.form_elements.content.content_video.settings'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'SIZES', 'SIZE_400x300', 'SIZE_500x400', 'DEFAULT_SIZE',
)

from .conf import get_setting

SIZE_400x300 = get_setting('SIZE_400x300')
SIZE_500x400 = get_setting('SIZE_500x400')
DEFAULT_SIZE = get_setting('DEFAULT_SIZE')
SIZES = get_setting('SIZES')



__title__ = 'fobi.forms'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'FormEntryForm', 'FormFieldsetEntryForm', 'FormElementEntryFormSet',
    'BulkChangeFormElementPluginsForm', 'BulkChangeFormHandlerPluginsForm',
)

from six.moves.urllib.parse import urlparse
import socket

from django.forms.models import modelformset_factory
from django import forms
from django.utils.translation import ugettext, ugettext_lazy as _
from fobi.models import (
    # Plugins
    FormElement, FormHandler,

    # Entries
    FormEntry, FormFieldsetEntry, FormElementEntry
    )
from fobi.constants import ACTION_CHOICES
from fobi.base import get_theme
from fobi.validators import url_exists
from fobi.exceptions import ImproperlyConfigured

# *****************************************************************************
# *****************************************************************************
# ******************************* Entry forms *********************************
# *****************************************************************************
# *****************************************************************************

class FormEntryForm(forms.ModelForm):
    """
    Form for ``fobi.models.FormEntry`` model.
    """
    class Meta:
        model = FormEntry
        fields = ('name', 'is_public', 'success_page_title',
                  'success_page_message', 'action',) #'is_cloneable',

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request', None)
        if self.request is None:
            raise ImproperlyConfigured(
                ugettext("The {0} form requires a"
                         "request argument.".format(self.__class__.__name__))
                )

        super(FormEntryForm, self).__init__(*args, **kwargs)
        theme = get_theme(request=None, as_instance=True)

        self.fields['name'].widget = forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
            )

        self.fields['success_page_title'].widget = forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
            )

        self.fields['success_page_message'].widget = forms.widgets.Textarea(
            attrs={'class': theme.form_element_html_class}
            )

        self.fields['action'].widget = forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
            )

        # At the moment this is done for Foundation 5 theme. Remove this once
        # it's possible for a theme to override this form. Alternatively, add
        # the attrs to the theme API.
        self.fields['is_public'].widget = forms.widgets.CheckboxInput(
            attrs={'data-customforms': 'disabled'}
            )
        #self.fields['is_cloneable'].widget = forms.widgets.CheckboxInput(
        #    attrs={'data-customforms': 'disabled'}
        #    )

    def clean_action(self):
        """
        Validate the action (URL). Checks if URL exists.
        """
        url = self.cleaned_data['action']
        if url:
            full_url = url

            if not (url.startswith('http://') or url.startswith('https://')):
                full_url = self.request.build_absolute_uri(url)

            parsed_url = urlparse(full_url)

            local = False

            try:
                localhost = socket.gethostbyname('localhost')
            except Exception as err:
                localhost = '127.0.0.1'

            try:
                host = socket.gethostbyname(parsed_url.hostname)

                local = (localhost == host)
            except socket.gaierror as err:
                pass

            if local:
                full_url = parsed_url.path

            if not url_exists(full_url, local=local):
                raise forms.ValidationError(
                    ugettext("Invalid action URL {0}.").format(full_url)
                    )

        return url


class FormFieldsetEntryForm(forms.ModelForm):
    """
    Form for ``fobi.models.FormFieldsetEntry`` model.
    """
    class Meta:
        model = FormFieldsetEntry
        fields = ('name',)

    def __init__(self, *args, **kwargs):
        super(FormFieldsetEntryForm, self).__init__(*args, **kwargs)
        theme = get_theme(request=None, as_instance=True)
        self.fields['name'].widget = forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
            )


FormElementEntryFormSet = modelformset_factory(
    FormElementEntry, fields=('position',), extra=0
    )

# *****************************************************************************
# *****************************************************************************
# *********************************** Base ************************************
# *****************************************************************************
# *****************************************************************************


class BaseBulkChangePluginsForm(forms.ModelForm):
    """
    Bulk change plugins form.

    - `selected_plugins` (str): List of comma separated values to be
       changed.
    - `users_action` (int): For indicating wheither the users shall be appended
      to the dashbard plugins or replaced.
    - `groups_action` (int): For indicating wheither the groups shall be
      appended to the dashboard plugins or replaced.
    """
    selected_plugins = forms.CharField(
        required=True, label=_("Selected plugins"),
        widget=forms.widgets.HiddenInput
        )
    users_action = forms.ChoiceField(
        required = False,
        label = _("Users action"),
        choices = ACTION_CHOICES,
        help_text = _("If set to ``replace``, the groups are replaced; "
                      "otherwise - appended.")
        )
    groups_action = forms.ChoiceField(
        required = False,
        label = _("Groups action"),
        choices = ACTION_CHOICES,
        help_text = _("If set to ``replace``, the groups are replaced; "
                      "otherwise - appended.")
        )

    class Media:
        css = {
            'all': ('css/admin_custom.css',)
        }

    def __init__(self, *args, **kwargs):
        super(BaseBulkChangePluginsForm, self).__init__(*args, **kwargs)
        self.fields['users'].required = False
        self.fields['groups'].required = False


class BulkChangeFormElementPluginsForm(BaseBulkChangePluginsForm):
    """
    """
    class Meta:
        model = FormElement
        fields = ['groups', 'groups_action', 'users', 'users_action',]


class BulkChangeFormHandlerPluginsForm(BaseBulkChangePluginsForm):
    """
    """
    class Meta:
        model = FormHandler
        fields = ['groups', 'groups_action', 'users', 'users_action',]



__title__ = 'fobi.context_processors'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('theme',)

from fobi.base import get_theme

def theme(request):
    """
    Gets active theme.

    :param django.http.HttpRequest request:
    :return fobi.base.BaseTheme: Instance of ``fobi.base.BaseTheme``.
    """
    return {'fobi_theme': get_theme(request, as_instance=True)}
